# encoding: UTF-8
class AutomatizacionObserver < ActiveRecord::Observer
  include UserInfo
  observe :gasto, :presupuesto, :actividad, :indicador, :fuente_verificacion, :documento

  def after_save(objeto)
    # Esto tendremos que cambiarlo cuando mezclemos comentarios y marcado para evitar que haya anotaciones dobles (comentario + marcado => comentario)
    # Si ha cambiado y no ha sido solo el marcado o la fecha de actualizacion...
    cambios = objeto.changed
    cambios.delete("marcado_id") if objeto.respond_to?('marcado_id')
    cambios.delete("updated_at") if objeto.respond_to?('updated_at')
    if cambios.size > 0
      marcar_elemento objeto if objeto.respond_to?('marcado')
      comentario_elemento(objeto,cambios) if current_user && objeto.respond_to?('comentario')
    end
  end

  # Marca el elemento cuando se ha modificado o creado
  def marcar_elemento(objeto)
    # Marca el elemento solo si no hemos hecho cambios en el marcado
    if objeto.marcado_id == objeto.marcado_id_was
      marc=Marcado.find_by_automatico(true)
      if marc
        # Guarda evitando callbacks 
        objeto.update_column(:marcado_id, marc.id)
      end
    end
  end

  # Genera un comentario automatico con la modificacion
  def comentario_elemento(objeto, cambios)
    # Comentario de que el elemento ha sido creado
    texto = objeto.class.name + " " + _("creado") if objeto.id_was.nil?
    # Comentario de elemento modificado (junto a los campos tocados)
    texto = objeto.class.name + " " + _("modificado") + " (" + _("campos") + ": " + cambios.join(", ") + ") " unless objeto.id_was.nil?
    objeto.comentario << Comentario.create(:usuario_id => current_user.id, :texto => texto)
  end
end

